"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteProject = exports.importProject = exports.openProject = exports.createProject = exports.getProject = exports.isValidName = exports.projectExists = exports.getProjects = void 0;
var fs_extra_1 = __importDefault(require("fs-extra"));
var path_1 = __importDefault(require("path"));
var config_1 = require("./config");
var paths_1 = require("./paths");
var commands_1 = require("./commands");
var logs_1 = require("./logs");
var child_process_1 = require("child_process");
var _loadProjects = loadProjects();
function loadProjects() {
    var projects;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                projects = fs_extra_1.default.readJSONSync(paths_1.PROJECTS);
                _a.label = 1;
            case 1:
                if (!true) return [3, 3];
                return [4, projects];
            case 2:
                _a.sent();
                return [3, 1];
            case 3: return [2];
        }
    });
}
function setProjects(projects) {
    fs_extra_1.default.writeFileSync(paths_1.PROJECTS, JSON.stringify(projects, undefined, 4));
    _loadProjects = loadProjects();
}
function addProject(newProject) {
    var updatedProjects = __spreadArray(__spreadArray([], getProjects()), [newProject]);
    setProjects(updatedProjects);
}
function updateProject(project) {
    removeProject(project.name);
    addProject(project);
}
function removeProject(projectName) {
    var updatedProjects = getProjects().filter(function (project) { return project.name !== projectName; });
    setProjects(updatedProjects);
}
function getProjects() {
    return _loadProjects.next().value;
}
exports.getProjects = getProjects;
function projectExists(projectName) {
    return getProjects().some(function (project) { return project.name === projectName; });
}
exports.projectExists = projectExists;
function isValidName(projectName) {
    var regex = new RegExp("^[a-zA-Z0-9_\\.\\-]+$");
    if (!projectName.match(regex))
        return false;
    return true;
}
exports.isValidName = isValidName;
function getProject(projectName) {
    var project = getProjects().find(function (project) { return project.name === projectName; });
    if (project == undefined) {
        throw new commands_1.OrigeenError("A project named: " + projectName + " was not found.", [
            "Check the spelling",
            "List all your projects: " + logs_1.bash("projects"),
        ]);
    }
    return project;
}
exports.getProject = getProject;
function isSubProject(pathToProject) {
    getProjects().forEach(function (_a) {
        var path = _a.path;
        if (path.includes(pathToProject) || pathToProject.includes(path)) {
            return true;
        }
    });
    return false;
}
function createProject(projectName, templateName) {
    if (templateName === void 0) { templateName = paths_1.EMPTY_TEMPLATE; }
    var debug = console.debug, log = console.log;
    log("Creating a project");
    log();
    log("Name: " + projectName);
    log("Template: " + templateName);
    log();
    if (!isValidName(projectName)) {
        throw new commands_1.OrigeenError("The project name is not valid.", [
            "Check that it is alpha-numerical. (it can still contain '-', '.', '_')",
        ]);
    }
    if (projectExists(projectName)) {
        throw new commands_1.OrigeenError("You can't create two projects with the name. '" + projectName + "' is already used.", [
            "Choose another name",
            "Check the spelling",
            "Delete the existing project",
        ]);
    }
    var pathToProject = path_1.default.resolve(config_1.getProperty("workspace"), projectName);
    debug("Absolute path to project:");
    debug(pathToProject);
    debug();
    if (fs_extra_1.default.existsSync(pathToProject)) {
        throw new commands_1.OrigeenError("It looks like a folder already exists at:\n" +
            (pathToProject + "\n") +
            "\n" +
            "Did you mean to import it?", [
            "Import/register the project into Origeen instead of creating it",
            "Delete the existing folder",
        ]);
    }
    if (isSubProject(pathToProject)) {
        throw new commands_1.OrigeenError("Origeen doesn't support sub-projects\n" +
            ("The given path '" + pathToProject + "' relates\nsomehow to '" + path_1.default + "' in a unsupported way.\n") +
            "Such a project structure would cause inconvenience.", [
            "Check the spelling of the path",
            "Think of another location for the project",
            "Delete the other project",
        ]);
    }
    var templatePath = path_1.default.join(paths_1.TEMPLATES, templateName);
    debug("Absolute path to template:");
    debug(templatePath);
    debug();
    debug("Copying template to the project destination");
    fs_extra_1.default.copySync(templatePath, pathToProject);
    debug("Copied!");
    debug();
    debug("Adding new project to 'projects.json'");
    addProject({
        name: projectName,
        path: pathToProject,
        createdAt: Date.now(),
        importedAt: -1,
        lastOpenedAt: -1,
        openings: 0,
    });
    debug("Added!");
    debug();
}
exports.createProject = createProject;
function openProject(projectName) {
    var debug = console.debug, log = console.log;
    var editorPath = config_1.getProperty("editorPath");
    var project = getProject(projectName);
    log("Opening project named: " + projectName);
    log();
    log("Path to project:");
    log(project.path);
    log();
    debug("Path to editor");
    debug(editorPath);
    debug();
    try {
        child_process_1.execFileSync(editorPath, [project.path], { encoding: "utf8" });
    }
    catch (error) {
        throw new commands_1.OrigeenError("Origeen tried to run your favourite code editor\n" +
            ("$ \"" + editorPath + "\" \"" + project.path + "\"\n") +
            "\n" +
            "However, an error occured.", ["Run the command manually", "Check the path to your editor"]);
    }
    project.openings++;
    updateProject(project);
}
exports.openProject = openProject;
function importProject(pathToProject) {
    var log = console.log;
    if (!path_1.default.isAbsolute(pathToProject)) {
        throw new commands_1.OrigeenError("You need to provide the absolute path to the project you're importing.", ["Re-run the command, but provide an absolute path"]);
    }
    if (!fs_extra_1.default.existsSync(pathToProject)) {
        throw new commands_1.OrigeenError("The path you provided doesn't exist on your machine.", [
            "Check the spelling",
            "Use an absolute path to the project you're importing, if that's not already the case.",
        ]);
    }
    if (isSubProject(pathToProject)) {
        throw new commands_1.OrigeenError("Origeen doesn't support sub-projects\n" +
            ("The given path '" + pathToProject + "' relates\nsomehow to '" + path_1.default + "' in a unsupported way.\n") +
            "Such a project structure would cause inconvenience.", [
            "Check the spelling of the path",
            "Think of another location for the project",
            "Delete the other project",
        ]);
    }
    var projectName = path_1.default.basename(pathToProject);
    log("Importing a project...");
    log("Name: " + projectName);
    log("Path: " + pathToProject);
    log();
    addProject({
        name: path_1.default.basename(pathToProject),
        path: pathToProject,
        createdAt: -1,
        importedAt: Date.now(),
        lastOpenedAt: -1,
        openings: 0,
    });
}
exports.importProject = importProject;
function deleteProject(projectName) {
    return __awaiter(this, void 0, void 0, function () {
        var log, warn, debug, group, groupEnd, project, verification, notSoSure;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    log = console.log, warn = console.warn, debug = console.debug, group = console.group, groupEnd = console.groupEnd;
                    project = getProject(projectName);
                    log("Deleting a project...");
                    log("Name: " + project.name);
                    log("Path: " + project.path);
                    log();
                    warn("You are about to delete a project!");
                    warn("You can't go back.");
                    warn();
                    warn("This will:");
                    group();
                    warn("- Delete the project recursively");
                    warn("- Unregister it from Origeen");
                    warn();
                    groupEnd();
                    return [4, logs_1.ask("Are you sure? ", logs_1.booleanValidator)];
                case 1:
                    verification = _a.sent();
                    notSoSure = verification.toLowerCase() === "n" ||
                        verification.toLowerCase() === "no"
                        ? true
                        : false;
                    if (notSoSure) {
                        throw new commands_1.OrigeenError("Project deletion cancelled.", [
                            "Re-run the command, but be sure you wan't to delete this project.",
                        ]);
                    }
                    debug("Deleting content on the disk...");
                    fs_extra_1.default.rmdirSync(project.path, { recursive: true });
                    debug("Deleted!");
                    debug("Unregistering the project from Origeen...");
                    removeProject(projectName);
                    debug("Unregistered!");
                    return [2];
            }
        });
    });
}
exports.deleteProject = deleteProject;
