"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.remove = exports.add = void 0;
var child_process_1 = require("child_process");
var fs_extra_1 = __importDefault(require("fs-extra"));
var path_1 = __importDefault(require("path"));
var commands_1 = require("./commands");
var paths_1 = require("./paths");
function add(templateName, pathOrUrl, remote) {
    var error = console.error;
    if (isReservedName(templateName)) {
        error("The template name '" + templateName + "' looks like a reserved name");
        error("You cannot create a template with this name.");
        error("Please retry with another name");
        throw new Error();
    }
    if (remote)
        return createTemplateFromRemote(pathOrUrl, templateName);
    createTemplateFromLocal(pathOrUrl, templateName);
}
exports.add = add;
function createTemplateFromLocal(source, templateName) {
    var debug = console.debug, error = console.error;
    var pathExists = fs_extra_1.default.existsSync(source);
    if (!pathExists) {
        throw new Error("The provided source doesn't exist.");
    }
    var destination = path_1.default.resolve(paths_1.TEMPLATES, templateName);
    debug("Destination folder :");
    debug(destination);
    debug();
    var templateExists = fs_extra_1.default.existsSync(destination);
    if (templateExists) {
        debug("Path " + destination + " already exists!");
        debug();
        error("This template name is already used. You can:");
        error("  - Delete the existing template");
        error("  - Choose another name");
        error();
        throw new Error("The template already exists !");
    }
    debug("Copying '" + source + "'");
    debug("To: '" + destination + "'");
    debug();
    fs_extra_1.default.copySync(source, destination);
}
function createTemplateFromRemote(source, templateName) {
    var error = console.error, debug = console.debug;
    var destination = path_1.default.join(paths_1.TEMPLATES, templateName);
    var command = "git clone \"" + source + "\" \"" + destination + "\"";
    try {
        debug("Running \"" + command + "\"");
        child_process_1.execSync(command);
    }
    catch (err) {
        throw new commands_1.OrigeenError("Unkown error while executing commmand", [
            "Check that 'git' is installed on your machine",
            "Make sure that '" + templateName + "' is not already used for another template"
        ]);
    }
    fs_extra_1.default.rmdirSync(path_1.default.join(destination, ".git"), { recursive: true });
}
function remove(templateName) {
    var error = console.error, debug = console.debug;
    if (isReservedName(templateName)) {
        error("You gave a template name beginning with '@'");
        error("Those are reserved template names and thus cannot be deleted");
        error();
        throw new Error("Naming error");
    }
    var pathToTemplate = path_1.default.join(paths_1.TEMPLATES, templateName);
    if (!fs_extra_1.default.existsSync(pathToTemplate)) {
        error("The given template name was not found.");
        error("Make sure you spelled it properly");
        error();
        throw new Error("Template not found");
    }
    debug("Removing '" + pathToTemplate + "' recursively");
    debug();
    fs_extra_1.default.rmdirSync(pathToTemplate, { recursive: true });
}
exports.remove = remove;
function isReservedName(name) {
    if (name.startsWith("@"))
        return true;
    return false;
}
function isValidName(name) {
    var regex = new RegExp("^[a-zA-Z0-9]+$");
    if (name.match(regex))
        return false;
    return true;
}
